<table class="display datatable" data-source-url="{{ url }}" data-ajax-method="{{ config.request_method }}"
       id="{{ config.id }}"
       data-result-counter-id="{{ config.result_counter_id }}"
       data-page-length="{{ config.page_length }}">
    <thead>
        <tr>
            {% for column in datatable %}
                {{ column }}
            {% endfor %}
        </tr>
    </thead>
    {% if config.footer %}
    <tfoot>
        <tr>
            {% for column in datatable %}
                <th>{{ column.label }}</th>
            {% endfor %}
        </tr>
    </tfoot>
    {% endif %}
</table>
