# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-10-05 02:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example_app', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='entry',
            name='is_published',
            field=models.BooleanField(default=False),
        ),
    ]
